/* VRANK pattern matching suite
 * Copyright (C) 1996 to Justin Zobel
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * See Copyright.txt for further details. */


#include <stdio.h>
#ifdef SOLARIS
#include <string.h>
#else /* SOLARIS */
#include <strings.h>
#endif /* SOLARIS */

#define true		1
#define false		0

#define STRLEN		60

int	edit3ex_old(char *, char *);
int	edit3extaper_old(char *, char *);
int	edit3ex(char *, char *);
int	edit3extaper(char *, char *);

static int min3(int,int,int);
static int reprate_old(char,char);
static int delrate_old(char,char);
static int reprate(char,char);
static int delrate(char,char);


#define SILENTGROUP 7

/* Soundex mappings:
    abcdefghijklmnopqrstuvwxyz
    01230120022455012623010202 */

/* My mappings:
    abcdefghijklmnopqrstuvwxyz
    01230127022455012623017202 */


#define SMALLP	2
#define BIGP	3


static int groupsof[128] = {
	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
	-1,  0,  1,  2,  3,  0,  1,  2,  7,  0,  2,  2,  4,  5,  5,  0,
	 1,  2,  6,  2,  3,  0,  1,  0,  2,  0,  2, -1, -1, -1, -1, -1,
	-1,  0,  1,  2,  3,  0,  1,  2,  0,  0,  2,  2,  4,  5,  5,  0,
	 1,  2,  6,  2,  3,  0,  1,  7,  2,  0,  2, -1, -1, -1, -1, -1 };


#define B BIGP
#define T SMALLP

int pairvals[27][27] = {
/*        a b c  d e f  g h i  j k l  m n o  p q r  s t u  v w x  y z X

/* a */	{ 0,B,B, B,T,B, B,B,T, B,B,B, B,B,T, B,B,B, B,B,T, B,B,B, T,B,B },
/* b */	{ B,0,B, B,B,B, B,B,B, B,B,B, B,B,B, T,B,B, B,B,B, B,B,B, B,B,B },
/* c */	{ B,B,0, B,B,B, B,B,B, B,T,B, B,B,B, B,T,B, T,B,B, B,B,B, B,T,B },

/* d */	{ B,B,B, 0,B,B, B,B,B, B,B,B, B,B,B, B,B,B, B,T,B, B,B,B, B,B,B },
/* e */	{ T,B,B, B,0,B, B,B,T, B,B,B, B,B,T, B,B,B, B,B,T, B,B,B, T,B,B },
/* f */	{ B,B,B, B,B,0, B,B,B, B,B,B, B,B,B, T,B,B, B,B,B, T,B,B, B,B,B },

/* g */	{ B,B,B, B,B,B, 0,B,B, T,B,B, B,B,B, B,B,B, B,B,B, B,B,B, B,B,B },
/* h */	{ B,B,B, B,B,B, B,0,B, B,B,B, B,B,B, B,B,B, B,B,B, B,B,B, B,B,B },
/* i */	{ T,B,B, B,T,B, B,B,0, B,B,B, B,B,T, B,B,B, B,B,T, B,B,B, T,B,B },

/* j */	{ B,B,B, B,B,B, T,B,B, 0,B,B, B,B,B, B,B,B, B,B,B, B,B,B, B,B,B },
/* k */	{ B,B,T, B,B,B, B,B,B, B,0,B, B,B,B, B,T,B, B,B,B, B,B,B, B,B,B },
/* l */	{ B,B,B, B,B,B, B,B,B, B,B,0, B,B,B, B,B,T, B,B,B, B,B,B, B,B,B },

/* m */	{ B,B,B, B,B,B, B,B,B, B,B,B, 0,T,B, B,B,B, B,B,B, B,B,B, B,B,B },
/* n */	{ B,B,B, B,B,B, B,B,B, B,B,B, T,0,B, B,B,B, B,B,B, B,B,B, B,B,B },
/* o */	{ T,B,B, B,T,B, B,B,T, B,B,B, B,B,0, B,B,B, B,B,T, B,B,B, T,B,B },

/* p */	{ B,T,B, B,B,T, B,B,B, B,B,B, B,B,B, 0,B,B, B,B,B, T,B,B, B,B,B },
/* q */	{ B,B,T, B,B,B, B,B,B, B,T,B, B,B,B, B,0,B, B,B,B, B,B,B, B,B,B },
/* r */	{ B,B,B, B,B,B, B,B,B, B,B,T, B,B,B, B,B,0, B,B,B, B,B,B, B,B,B },

/* s */	{ B,B,T, B,B,B, B,B,B, B,B,B, B,B,B, B,B,B, 0,B,B, B,B,T, B,T,B },
/* t */	{ B,B,B, T,B,B, B,B,B, B,B,B, B,B,B, B,B,B, B,0,B, B,B,B, B,B,B },
/* u */	{ T,B,B, B,T,B, B,B,T, B,B,B, B,B,T, B,B,B, B,B,0, B,B,B, T,B,B },

/* v */	{ B,B,B, B,B,T, B,B,B, B,B,B, B,B,B, T,B,B, B,B,B, 0,B,B, B,B,B },
/* w */	{ B,B,B, B,B,B, B,B,B, B,B,B, B,B,B, B,B,B, B,B,B, B,0,B, B,B,B },
/* x */	{ B,B,B, B,B,B, B,B,B, B,B,B, B,B,B, B,B,B, T,B,B, B,B,0, B,T,B },

/* y */	{ T,B,B, B,T,B, B,B,T, B,B,B, B,B,T, B,B,B, B,B,T, B,B,B, 0,B,B },
/* z */	{ B,B,T, B,B,B, B,B,B, B,B,B, B,B,B, B,B,B, T,B,B, B,B,T, B,0,B },
/* X */ { B,B,B, B,B,B, B,B,B, B,B,B, B,B,B, B,B,B, B,B,B, B,B,B, B,B,B }
};


/* Phonetic edit distance: cost of insertion, deletion, swap is SMALLP
   if in the same group, BIGP otherwise.  Adjacent identical letters
   can be deleted for free.  */
int
edit3ex_old(char *in1, char *in2)
{
    int		F[STRLEN+1][STRLEN+1];
    int		i, j, l1, l2;
    int		inc1, inc2, diag;
    int		sum;

    l1 = strlen(in1);
    l2 = strlen(in2);

    F[0][0] = 0;
    F[1][0] = BIGP;
    for( sum=BIGP, i=2 ; i<=l1 ; i++ )
    {
	sum += delrate_old(in1[i-2], in1[i-1]);
	F[i][0] = sum;
    }
    F[0][1] = BIGP;
    for( sum=BIGP, j=2 ; j<=l2 ; j++ )
    {
	sum += delrate_old(in2[j-2], in2[j-1]);
	F[0][j] = sum;
    }
    for( i=1 ; i<=l1 ; i++ )
    {
	for( j=1 ; j<=l2 ; j++ )
	{
	    inc1 = ( i==1 ) ? BIGP : delrate_old(in1[i-2], in1[i-1]);
	    inc2 = ( j==1 ) ? BIGP : delrate_old(in2[j-2], in2[j-1]);
	    diag = reprate_old(in1[i-1], in2[j-1]);

	    F[i][j] = min3(F[i-1][j] + inc1,
			      F[i][j-1] + inc2,
			      F[i-1][j-1] + diag);
	}
    }
    
    return((int) F[l1][l2]);
}


/* Phonetic edit distance: cost of insertion, deletion, swap is SMALLP
   if in the same group, BIGP otherwise.  Adjacent identical letters
   can be deleted for free.  */
int
edit3extaper_old(char *in1, char *in2)
{
    int	F[STRLEN+1][STRLEN+1];
    int		i, j, l1, l2;
    int		inc1, inc2, diag;
    int		sum, maxp;

    l1 = strlen(in1);
    l2 = strlen(in2);

    maxp = 2 * ( l1 + l2 );

    F[0][0] = 0;
    for( sum=0, i=1 ; i<=l1 ; i++ )
    {
	sum += ( maxp-i ) * ( ( i==1 ) ? BIGP : delrate_old(in1[i-2], in1[i-1]) );
	F[i][0] = sum;
    }
    for( sum=0, i=1 ; i<=l2 ; i++ )
    {
	sum += ( maxp-i ) * ( ( i==1 ) ? BIGP : delrate_old(in2[i-2], in2[i-1]) );
	F[0][i] = sum;
    }

    for( i=1 ; i<=l1 ; i++ )
    {
	for( j=1 ; j<=l2 ; j++ )
	{
	    inc1 = ( maxp-i-j ) * ( ( i==1 ) ? BIGP : delrate_old(in1[i-2], in1[i-1]) );
	    inc2 = ( maxp-i-j ) * ( ( j==1 ) ? BIGP : delrate_old(in2[j-2], in2[j-1]) );
	    diag = ( maxp-i-j ) * reprate_old(in1[i-1], in2[j-1]);

	    F[i][j] = min3(F[i-1][j] + inc1,
			      F[i][j-1] + inc2,
			      F[i-1][j-1] + diag);
	}
    }
    
    return((int) F[l1][l2]);
}


/* cost of a replacement */
int
reprate_old(char c1, char c2)
{
    if( c1 == c2 )
	return(0);
    else if( groupsof[c1] == groupsof[c2] )
	return(SMALLP);
    else
	return(BIGP);
}


/* cost of a deletion */
int
delrate_old(char c1, char c2)
{
    if( c1 == c2 )
	return(0);
    else if( groupsof[c1] == groupsof[c2] || groupsof[c2] == SILENTGROUP )
	return(SMALLP);
    else
	return(BIGP);
}


/* Phonetic edit distance: cost of insertion, deletion, swap is based on the
   array pairvals.  Adjacent identical letters can be deleted for free.  */
int
edit3ex(char *in1, char *in2)
{
    int		F[STRLEN+1][STRLEN+1];
    int		i, j, l1, l2;
    int		inc1, inc2, diag;
    int		sum;
    char	s1[STRLEN], s2[STRLEN];

    l1 = strlen(in1);
    l2 = strlen(in2);

	/* manipulate strings to simplify comparison */
    for( i=0 ; i<l1 ; i++ )
	s1[i] = ( isalpha(in1[i]) ) ? tolower(in1[i]-'a') : 26;
    s1[i] = '\0';
    for( j=0 ; j<l2 ; j++ )
	s2[j] = ( isalpha(in2[j]) ) ? tolower(in2[j]-'a') : 26;
    s2[j] = '\0';

    F[0][0] = 0;
    F[1][0] = BIGP;
    for( sum=BIGP, i=2 ; i<=l1 ; i++ )
    {
	sum += delrate(s1[i-2], s1[i-1]);
	F[i][0] = sum;
    }
    F[0][1] = BIGP;
    for( sum=BIGP, j=2 ; j<=l2 ; j++ )
    {
	sum += delrate(s2[j-2], s2[j-1]);
	F[0][j] = sum;
    }
    for( i=1 ; i<=l1 ; i++ )
    {
	for( j=1 ; j<=l2 ; j++ )
	{
	    inc1 = ( i==1 ) ? BIGP : delrate(s1[i-2], s1[i-1]);
	    inc2 = ( j==1 ) ? BIGP : delrate(s2[j-2], s2[j-1]);
	    diag = reprate(s1[i-1], s2[j-1]);

	    F[i][j] = min3(F[i-1][j] + inc1,
			      F[i][j-1] + inc2,
			      F[i-1][j-1] + diag);
	}
    }
    
    return((int) F[l1][l2]);
}


/* Phonetic edit distance: cost of insertion, deletion, swap is based on the
   array pairvals.  Adjacent identical letters can be deleted for free.  */
int
edit3extaper(char *in1, char *in2)
{
    int		F[STRLEN+1][STRLEN+1];
    int		i, j, l1, l2;
    int		inc1, inc2, diag;
    int		sum, maxp;
    char	s1[STRLEN], s2[STRLEN];

    l1 = strlen(in1);
    l2 = strlen(in2);

    maxp = 2 * ( l1 + l2 );

	/* manipulate strings to simplify comparison */
    for( i=0 ; i<l1 ; i++ )
	s1[i] = ( isalpha(in1[i]) ) ? tolower(in1[i]-'a') : 26;
    s1[i] = '\0';
    for( j=0 ; j<l2 ; j++ )
	s2[j] = ( isalpha(in2[j]) ) ? tolower(in2[j]-'a') : 26;
    s2[j] = '\0';

    F[0][0] = 0;
    sum = F[1][0] = ( maxp-1 ) * BIGP;
    for( i=2 ; i<=l1 ; i++ )
    {
	sum += ( maxp-j ) * delrate(s1[i-2], s1[i-1]);
	F[i][0] = sum;
    }
    sum = F[0][1] = ( maxp-1 ) * BIGP;
    for( j=2 ; j<=l2 ; j++ )
    {
	sum += ( maxp-j ) * delrate(s2[j-2], s2[j-1]);
	F[0][j] = sum;
    }
    for( i=1 ; i<=l1 ; i++ )
    {
	for( j=1 ; j<=l2 ; j++ )
	{
	    inc1 = ( maxp-i-j ) * ( ( i==1 ) ? BIGP : delrate(s1[i-2], s1[i-1]) );
	    inc2 = ( maxp-i-j ) * ( ( j==1 ) ? BIGP : delrate(s2[j-2], s2[j-1]) );
	    diag = ( maxp-i-j ) * reprate(s1[i-1], s2[j-1]);

	    F[i][j] = min3(F[i-1][j] + inc1,
			      F[i][j-1] + inc2,
			      F[i-1][j-1] + diag);
	}
    }
    
    return((int) F[l1][l2]);
}


/* cost of a replacement */
int
reprate(char c1, char c2)
{
    return( pairvals[c1][c2] );
}


/* cost of a deletion */
int
delrate(char c1, char c2)
{
    if( c1 == c2 )
	return(0);
    else if( groupsof[c2+'a'] == SILENTGROUP )
	return(SMALLP);
    else
	return( pairvals[c1][c2] );
}


int
min3(int a, int b, int c)
{
    if( a > b )
	return( ( c > b ) ? b : c );
    else
	return( ( c > a ) ? a : c );
}
