/* VRANK pattern matching suite
 * Copyright (C) 1996 to Justin Zobel
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * See Copyright.txt for further details. */


/* Hash table for maintaining grams */

#include "def.h"

static int hash(char *);

#define TABLESIZE	25000
#define NPRIMES		20

static int	primes[] = { 101749, 190763, 165877, 190097, 84673,
			     193357, 157721, 102397, 103837, 197803,
			     190753, 200323, 101111, 111659, 222029,
			     14347, 92219, 23053, 77999, 220373 };

static HNODE	**table;
static SNODE	**stable;
static int htablesize=0;

/* Initialise the hash table for the normal case*/
void
inithash(int tabsize)
{
    int		i;

    htablesize = tabsize;
    table = (HNODE **) malloc(sizeof(HNODE *) * htablesize);

    for( i=0 ; i<htablesize ; i++ )
	table[i] = NULL;
}


/* Hash function to compute the hash value of a gram*/
int
hash(char *s)
{
    int		i, hval;

    for( i=hval=0 ; i < GRAMLEN ; i++ )
	hval += primes[i % NPRIMES]*s[i];
    return(hval % htablesize);
}


/* Insert gram into chained hash table. */
HNODE *
hinsert(char *gram)
{
    HNODE	*tmp;
    int		hval = hash(gram);

    tmp = (HNODE *) malloc(sizeof(HNODE));
    strncpy(tmp->gram, gram, GRAMLEN);
    tmp->next = table[hval];
    table[hval] = tmp;

    return(tmp);
}



/* Find gram in a hash table. */
HNODE *
hfetch(char *gram)
{
    int		hval = hash(gram);
    HNODE	*hp;

    for( hp=table[hval] ; hp != NULL &&
		strncmp(gram, hp->gram, GRAMLEN) != 0 ; hp = hp->next )
	;
    return(hp);
}


/* Find size of table -- ie, number of entries */
int
tablesize()
{
    int		i, c = 0;
    HNODE	*hp;

    for( i=0 ; i<htablesize ; i++ )
	for( hp=table[i] ; hp != NULL ; hp = hp->next )
	    c++;
    return(c);
}


/* Returns a pointer to a inverted file vector*/
BITVEC *
gethvec(HNODE *hp)
{
    return(&(hp->vec));
}


/* Initialise the hash table of nodes : used when skip is activated*/
void
sinithash(int tabsize)
{
    int         i;

    htablesize = tabsize;
    stable = (SNODE **) malloc(sizeof(SNODE *) * htablesize);

    for( i=0 ; i<htablesize ; i++ )
        stable[i] = NULL;
}


/* Insert gram into chained hash table. */
SNODE *
hsinsert(char *gram)
{
    SNODE       *tmp;
    int         hval = hash(gram);

    tmp = (SNODE *) malloc(sizeof(SNODE));
    strncpy(tmp->gram, gram, GRAMLEN);
    /*tmp->skiplen = sklen;*/
    tmp->next = stable[hval];
    stable[hval] = tmp;

    return(tmp);
}



/* Find gram in a hash table. */
SNODE *
hsfetch(char *gram)
{
    int         hval = hash(gram);
    SNODE       *hp;

    for( hp=stable[hval] ; hp != NULL &&
                strncmp(gram, hp->gram, GRAMLEN) != 0 ; hp = hp->next )
        ;
    return(hp);
}


/* Find size of table -- ie, number of entries */
int
stablesize()
{
    int         i, c = 0;
    SNODE       *hp;

    for( i=0 ; i<htablesize ; i++ )
        for( hp=stable[i] ; hp != NULL ; hp = hp->next )
            c++;
    return(c);
}


/* Returns a pointer to the inverted file vector*/
BITVEC *
getshvec(SNODE *hp)
{
    return(&(hp->vec));
}


/* Returns a pointer to the skip list vector*/
BITVEC *
getspvec(SNODE *hp)
{
    return(&(hp->svec));
}


/*Returns a pointer to skip list addresses' vector*/
BITVEC *
getsapvec(SNODE *hp)
{
    return(&(hp->saddrvec));
}



void freehash(void)
{
    int i;
    
    for(i=0 ; i < htablesize ; i++)
	free(table[i]);
    free(table);
}


void sfreehash(void)
{
    int i;
        
    for(i=0 ; i < htablesize ; i++)
        free(stable[i]);
    free(stable);
}
