/* VRANK pattern matching suite
 * Copyright (C) 1996 to Justin Zobel
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * See Copyright.txt for further details. */


#include <stdio.h>
#include <ctype.h>

int	isvowel(char);


/* Knuth's soundex algorithm */

/* Standard mapping:
   aehiouwy -> 0, which is omitted
   bfpv -> 1
   cgjkqsxz -> 2
   dt -> 3
   l -> 4
   mn -> 5
   r ->6
*/
/*                abcdefghijklmnopqrstuvwxyz */
#define KMAPPING "01230120022455012623010202"

void
soundex(char *in, int inlen, char *out)
{
    register int    c, lc, prev;
    register int    i;

    strcpy(out, "a000");
    prev = '0';

    for( i=0 ; inlen > 0 && i < 4 ; in++, inlen-- )
    {
	if( isalpha(*in) )
	{
	    lc = isupper(*in) ? tolower(*in) : *in;
	    c = KMAPPING[lc-'a'];
	    if ( i == 0 || ( c != '0' && c != prev ) )
	    {
		out[i] = i ? c : lc;
		i++;
	    }
	    prev = c;
	}
    }
    /* out[i] = '\0'; */
}

/* Another possible mapping:
   aehiouy -> 0, which is omitted
   bfpv -> 1
   cksxz -> 2
   dt -> 3
   l -> 4
   mn -> 5
   r ->6
   w -> 7
   gj -> 8
   q -> 9
#define NMAPPING "abcdabgaagclmmabqrcdabwcac"
or, equivalently
#define NMAPPING "01230180082455019623017202" */


/* Nontruncating soundex */
void
ntsoundex(char *in, int inlen, char *out)
{
    register int    c, lc, prev;
    register int    i;

    prev = '0';

    for( i=0 ; inlen > 0 ; in++, inlen-- )
    {
	if( isalpha(*in) )
	{
	    lc = isupper(*in) ? tolower(*in) : *in;
	    c = KMAPPING[lc-'a'];
	    if ( i == 0 || ( c != '0' && c != prev ) )
	    {
		out[i] = i ? c : lc;
		i++;
	    }
	    prev = c;
	}
    }
    out[i] = '\0';
}


/* Alan Kent's soundex algorithm: remove vowels, double letters */
/* Equivalent to [kn]soundex with mapping "abcdafghajklmnapqrstavwxaz" */
void
asoundex(char *in, int inlen, char *out)
{
    int		i;
    char	*p, *q;
    char	*f, *t;

    for( p=in, q=out, i=0 ; i < inlen ; p++, q++, i++ )
	*q = isupper(*p) ? tolower(*p) : *p;
    *q = '\0';
    t = out;
    for( f = out ; *f != '\0' ; f++)
    {
	    /* note: never remove first character */
	if( f == out || (!isvowel(*f) && *f != *(f-1)) )
	    *t++ = *f;
    }
    *t = '\0';
}


int
isvowel(char c)
{
    return( c == 'a' || c == 'e' ||  c == 'i'
	|| c == 'o' || c == 'u' || c == 'y' );
}
