/* VRANK pattern matching suite
 * Copyright (C) 1996 to Justin Zobel
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * See Copyright.txt for further details. */


#include "def.h"

void	ntsoundex(char *,int,char *);
void	soundex(char *,int,char *);
void	asoundex(char *,int,char *);
void	phonix(char *,int,char *);
void	ntphonix(char *,int,char *);
void	partphonix(char *,int,char *);

void
phonetic(int type, char *in, int inlen, char *out)
{
    switch( type )
    {
	case NOSOUNDEX :
	    strcpy(out, in);
	    break;
	case SOUNDEX :
	    soundex(in, inlen, out);
	    break;
	case NTSOUNDEX :
	    ntsoundex(in, inlen, out);
	    break;
	case ASOUNDEX :
	    asoundex(in, inlen, out);
	    break;
	case PHONIX :
	    phonix(in, inlen, out);
	    break;
	case NTPHONIX :
	    ntphonix(in, inlen, out);
	    break;
	case PARTPHONIX :
	    partphonix(in, inlen, out);
	    break;
	default :
	    fprintf(stderr, "Unrecognised phonetic routine %d.\n", type);
	    exit(1);
	    break;
    }
}


char *
phonetictype(int type)
{
    switch( type )
    {
	case NOSOUNDEX :
	    return("none");
	    break;
    	case SOUNDEX :
	    return("soundex");
	    break;
    	case NTSOUNDEX :
	    return("nt-sdex");
	    break;
    	case ASOUNDEX :
	    return("Atlas");
	    break;
    	case PHONIX :
	    return("phonix");
	    break;
    	case NTPHONIX :
	    return("nt-pix");
	    break;
    	case PARTPHONIX :
	    return("pt-pix");
	    break;
	default :
	    return(NULL);
	    break;
    }
}


char *
ranktype(int type)
{
    switch( type )
    {
	case PLAIN :
	    return("plain");
	    break;
	case QGRAM :
	    return("q-gram");
	    break;
    	case EDIT3 :
	    return("edit3");
	    break;
    	case EDIT3TAPER :
	    return("edit3t");
	    break;
    	case EDIT4 :
	    return("edit4");
	    break;
    	case PRIORITY :
	    return("priority");
	    break;
    	/* case EDITRAW :
	    return("rawedit");
	    break; */
    	case EDITEX :
	    return("edit3ex");
	    break;
    	case EDITEXTAPER :
	    return("ed3extp");
	    break;
    	case ONECHAR :
	    return("onechar");
	    break;
    	case LCS :
	    return("lcs");
	    break;
	default :
	    return(NULL);
	    break;
    }
}


int
validchar(char c)
{
	/* other characters break the phonetic transformation routines */
    /* return( isalnum(c) || c=='#' || c=='-' || c=='\'' ); */
    return( isalnum(c) || c=='#' );
}
